#ifndef AVL_H
#define AVL_H
// ---------------------------------------------------------------------------------------------------

#include "avlnode.h"
// ---------------------------------------------------------------------------------------------------

#define LEFT_CHILD 0
#define RIGHT_CHILD 1

#define REM_OP_TYPE_I 1
#define REM_OP_TYPE_II 2
#define REM_OP_TYPE_IIIA 3
#define REM_OP_TYPE_IIIB 4
#define REM_OP_TYPE_IIIC 5
// ---------------------------------------------------------------------------------------------------

class avl
{
private:

     avlnode* _root;              ///< korzen drzewa
     
     /// \brief Pomocnicza funkcja wstawiajaca wartosc
     ///
     /// \param __root - korzen podrzewa do ktorego nalezy wstawic wartosc
     /// \param __value - wartosc elementu do wstawienia
     /// \param __modChild - przy powrocie z rekurencji zawiera wskaznik zmodyfikowanego rodzica
     /// \retuerns true - jezeli wysokosc drzewa sie zwiekszyla i nalezy poprawiac wagi, inaczej false
     bool insertHelper(avlnode* __root, int __value, avlnode*& __modChild);
     
     /// \brief Metoda wyszukujaca element o zadanej wartosci
     ///
     /// \param __key - wartosc elementu
     /// \returns wskaznik do elementu o danej wartosci lub NULL jezeli nie istnieje
     avlnode* findNode(int __key);
     
     /// \brief Wyszukuje poprzednika (w przeszukiwaniu LVR) dla danego wezla 
     ///
     /// \param __node - wezel ktorego poprzednika chcemy wyszukac
     /// \returns wskaznik do wezla bedacego poprzednikiem wezla podanego jako argument
     avlnode* predescor(avlnode* __node);
     
     /// \brief Metoda wyszukujaca rodzica danego wezla
     ///
     /// \param __node - wskaznik do wezla ktorego rodzica chcemy wyszukac
     /// \returns wskaznik do wezla bedacego rodzicem wezla podanego jako argument
     avlnode* parent(avlnode* __node);
     
     /// \brief Pomocnicza metoda wyszukiwania rodzica danego wezla
     ///
     /// \param __node - aktualnie sprawdzany wezel
     /// \param __parent - arg. in/out. Jako wejsciowy jest wskaznikiem na wezel ktorego rodzica chcemy odnalezc, jako wyjsciowy jest wskaznikiem na rodzica tego wezla
     /// \returns true jezeli rodzic zostal znaleziony, inaczej false (np. dla root-a)
     bool parentHelper(avlnode* __node, avlnode*& __parent);
     
     /// \brief Pomocnicza metoda wyswietlajaca drzewo na ekranie
     ///
     /// \param __a - konfiguracja wyswietlania galezi
     /// \param __node - aktualnie odwiedzany wezel
     /// \param __indentSize - bierzacy poziom zagniezdzenia
     void printHelper(bool* __a, avlnode* __node, int __indentSize);
     
     // ROTACJE
     
     /// \brief Wykonuje pojedyncza rotacje w lewo
     ///
     /// \param __node - wezel wzgledem ktorego jest wykonywana rotacja
     /// \returns wskaznik do wezla ktory wejdzie na miesce tego podnaego jako argument
     avlnode* rotRR(avlnode* __node);
     
     /// \brief Wykonuje pojedyncza rotacje w prawo
     ///
     /// \param __node - wezel wzgledem ktorego jest wykonywana rotacja
     /// \returns wskaznik do wezla ktory wejdzie na miesce tego podnaego jako argument
     avlnode* rotLL(avlnode* __node);
     
     /// \brief Wykonuje podwujna rotacje prawo-lewo
     ///
     /// \param __node - wezel wzgledem ktorego jest wykonywana rotacja
     /// \returns wskaznik do wezla ktory wejdzie na miesce tego podnaego jako argument
     avlnode* rotRL(avlnode* __node);
     
     /// \brief Wykonuje podwujna rotacje lewo-prawo
     ///
     /// \param __node - wezel wzgledem ktorego jest wykonywana rotacja
     /// \returns  wskaznik do wezla ktory wejdzie na miesce tego podnaego jako argument
     avlnode* rotLR(avlnode* __node);
     
     /// \brief Wykonuje odpowiedni zestaw rotacji dla danego wezla
     ///
     /// \param __node - wezel wzgledem ktorego jest sprawdzany wykonywane sa rotacje
     /// \param __node -  wskaznik do wezla ktory wejdzie na miesce tego podnaego jako argument
     avlnode* fixBalance(avlnode* __node);
     
     /// \brief Wykonuje odpowiedni zestaw operacji dla danego wezla po operacji usuwania
     ///
     /// \param __node - wezel wzgledem ktorego jest sprawdzany wykonywane sa operacje
     /// \param __whichChild - okresla ktore podrzewo zostalo zmodyfikowane
     void fixBalanceAfterRemove(avlnode* __node, int __whichChild);

public:

     /// \brief Konstrutkor
     avl(void);

     /// \brief Destruktor
     ~avl(void);
     
     /// \brief Wstawia do drzewa nowy element
     ///
     /// \param __value - wartosc nowego elementu
     void insert(int __value);
     
     /// \brief Usuwa z drzewa element o zadanej wartosci
     ///
     /// \param __key - wartosc elementu do usuniecia
     void remove(int __key);
     
     /// \brief Usuwa z drzewa bezposredni poprzednik danego wezla
     ///
     /// \param __node - wskaznik na nastepnika
     /// \returns wartosc klucza wezla poprzednika
     int removePredescor(avlnode* __node);
     
     /// \brief Usuwa z drzewa element o zadanej wartosci
     ///
     /// \param __key - wartosc elementu do usuniecia
     void remove1(int __key);
     
     /// \brief Wyswietla drzewo na ekranie
     void print(void);
};
// -----------------------------------------------------------------------------

#endif
