#ifndef BINARY_HEAP_H
#define BINARY_HEAP_H
// -----------------------------------------------------------------------------

#define NEG_INF -100000000
// -----------------------------------------------------------------------------

class binary_heap
{
private:
     int* _valtab;    ///< Tablica wartosci
     int _size;          ///< Rozmiar tablicy
     
     /// \brief Zamienia miejscami dwie wartosci o podanych indeksach
     ///
     /// \param __index1 - indeks pierwszej wartosci
     /// \param __index2 - indeks drugiej wartosci
     void swap(int __index1, int __index2);
     
     /// \brief Przesuwa wszystkie elementy o jedno miejsce w lewo
     void shift(void);
     
     /// \brief Zmienia (powieksza, zminiesjza) wielkosc kopca o jeden element
     ///
     /// \param __increase - jezeli true to kopiec powinin zostac powiekszony
     void resizeHeap(bool __increase);
     
     /// \brief Kasuje dane kopca
     void clear(void);
     
     /// \brief Oblicza glebokosc (wysokosc) kopca
     ///
     /// \returns glebokosc (wysokosc) kopca
     int height(void);
     
     /// \brief Pomocnicza funckcja wypisujaca zawartosc kopca
     ///
     /// \param __a - tablica pomocnicza w wyswietlaniu drzewa
     /// \param __index - indeks wezla
     /// \param __indent - wielkosc wciecia
     void printHelper(bool* __a, int __index, int __indent);
     
     /// \brief Zwraca rodzica danego elementu - indeks w tabicy _valtab
     ///
     /// \param indeks elementu ktorego rodzica chcemy wyszukac
     /// \returns indeks rodzica, jezeli element nie ma rodzica to -1
     int parent(int __index);
     
     /// \brief Zwraca lewego potomka danego elementu - indeks w tabicy _valtab
     ///
     /// \param indeks elementu ktorego potomka chcemy wyszukac
     /// \returns indeks potomka, jezeli element nie ma lewego potomka to -1
     int left(int __index);
     
     /// \brief Zwraca prawego potomka danego elementu - indeks w tabicy _valtab
     ///
     /// \param indeks elementu ktorego potomka chcemy wyszukac
     /// \returns indeks potomka, jezeli element nie ma prawego potomka to -1
     int right(int __index);
     
     /// \brief Przywraca wlasnosc kopca, ktorego struktura zostala zaburzona w elemencie __index. Zaklada sie ze podrzewo tego elelentu jest juz poprawnym kopcem
     ///
     /// \param __index - indeks elementu ktory zaburza wlasnosc kopca
     void heapify(int __index);
     
     /// \brief Metoda pozwalajaca zwiekszyc wartosc wezla
     ///
     /// \param __index - elementu ktorego wartosc chcemy zwiekszyc
     /// \param __newvalue - nowa wartosc elementu
     void increase(int __index, int __newvalue);
     
public:
     
     /// \brief Konstrutor
     binary_heap(void);

     /// \brief Destruktor
     ~binary_heap(void);
     
     /// \brief Zwraca liczbe wszystkich elementow w tablicy
     ///
     /// \returns liczbe wszystkich elementow w tablicy
     int size(void);
     
     /// \brief Buduje kopiec z nieuporzadkownaje tablicy
     ///
     /// \param __array - tablica danych (nieuporzadkowana)
     /// \param __size - rozmiar tablicy
     void buildHeap(const int* __array, int __size);

     /// \brief Wstawia wartosc do  kopca
     ///
     /// \param __val - wartosc ktora wstawiamy
     void insert(int __val);

     /// \brief Usuwa wartosc maksymalna z kopca
     ///
     /// \returns usunieta maksymalna wartosc kopca
     int extract(void);
     
     /// \brief Zwraca wartosc maksymalna
     ///
     /// \returns zwraca wartosc maksymalna
     int maximum(void);
     
     /// \brief Zwraca wskaznik do tablicy posortowanej metoda heapsort
     ///
     /// \returns wskaznik do tablicy posortowanej metoda heapsort
     int* sort(void);

     /// \brief Wyswietla zawartosc kopca
     void print(void);
};
// -----------------------------------------------------------------------------

#endif
